/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Scale.h>
#include <Xm/List.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/SelectioB.h>
#include <Xm/FileSB.h>
#include <Mrm/MrmPublic.h>

#include "monitor.hxx"
#include "yoo_widget.hxx"


static ListVal<char*> ListBases, ListServers, ListClients ;
static ListVal<InfoClient> ListInfoClients ;
static char* CurrBase ;

#define BASE_LIST_WIDGET "*base_WL"
#define SERVER_LIST_WIDGET "*server_WL"
#define CLIENT_LIST_WIDGET "*client_WL"

void monitor_init()
{
  SvcInit() ;
  ListBases = list_bases() ;
  YOO_WidgetControl->set_choice_text(BASE_LIST_WIDGET, ListBases) ;
}


void monitor_select_baseCB()
{
  ListVal<int> res = YOO_WidgetControl->get_choice_selection(BASE_LIST_WIDGET);
  if (res.count() > 0) {
    CurrBase = ListBases[res[0]] ;
  } else {
    return ;
  }

  ListServers = list_servers(CurrBase) ;
  YOO_WidgetControl->set_choice_text(SERVER_LIST_WIDGET, ListServers) ;
  ListInfoClients = list_clients(CurrBase) ;
  ListClients.clear() ;
  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    char buff[512] ;
    sprintf(buff, "%s : %d", iter.get().name(), iter.get().pid()) ;    
    ListClients += buff ;
  }
  YOO_WidgetControl->set_choice_text(CLIENT_LIST_WIDGET, ListClients) ;
}


void monitor_select_serverCB()
{
  char* currServer ;

  ListVal<int> res=YOO_WidgetControl->get_choice_selection(SERVER_LIST_WIDGET);
  if (res.count() > 0) {
    currServer = ListServers[res[0]] ;
  } else {
    return ;
  }

  ListClients.clear() ;
  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    char buff[512] ;
    IterListVal<char*> iterS(iter.get().servers()) ;
    for (; iterS.more(); iterS.next()) {
      if (strcmp(iterS.get(), currServer) == 0) {
	sprintf(buff, "%s : %d", iter.get().name(), iter.get().pid()) ;    
	ListClients += buff ;
      }
    }
  }
  YOO_WidgetControl->set_choice_text(CLIENT_LIST_WIDGET, ListClients) ;
}


void monitor_select_clientCB()
{
  char* currClient ;
  
  ListVal<int> res=YOO_WidgetControl->get_choice_selection(CLIENT_LIST_WIDGET);
  if (res.count() > 0) {
    currClient = ListClients[res[0]] ;
  } else {
    return ;
  }

  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    if (strcmp(currClient, iter.get().name()) == 0) {
      ListServers = iter.get().servers() ;
      YOO_WidgetControl->set_choice_text(SERVER_LIST_WIDGET, ListServers) ;
      return ;
    }
  }

}


void monitor_kill_serverCB()
{
  ListVal<int> res=YOO_WidgetControl->get_choice_selection(SERVER_LIST_WIDGET);
  if (res.count() > 0) {
    kill_server(CurrBase, ListServers[res[0]]) ;
  } else {
    return ;
  }

  ListServers = list_servers(CurrBase) ;
  YOO_WidgetControl->set_choice_text(SERVER_LIST_WIDGET, ListServers) ;
  ListInfoClients = list_clients(CurrBase) ;
  ListClients.clear() ;
  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    char buff[512] ;
    sprintf(buff, "%s : %d", iter.get().name(), iter.get().pid()) ;    
    ListClients += buff ;
  }
  YOO_WidgetControl->set_choice_text(CLIENT_LIST_WIDGET, ListClients) ;
}


void monitor_kill_all_serversCB()
{
  kill_all_servers(CurrBase) ;

  ListServers = list_servers(CurrBase) ;
  YOO_WidgetControl->set_choice_text(SERVER_LIST_WIDGET, ListServers) ;
  ListInfoClients = list_clients(CurrBase) ;
  ListClients.clear() ;
  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    char buff[512] ;
    sprintf(buff, "%s : %d", iter.get().name(), iter.get().pid()) ;    
    ListClients += buff ;
  }
  YOO_WidgetControl->set_choice_text(CLIENT_LIST_WIDGET, ListClients) ;
}


void monitor_kill_clientCB()
{
  char* currClient ;

  ListVal<int> res=YOO_WidgetControl->get_choice_selection(CLIENT_LIST_WIDGET);
  if (res.count() > 0) {
    currClient = ListClients[res[0]] ;
  } else {
    return ;
  }

  IterListVal<InfoClient> iter(ListInfoClients) ;
  for (; iter.more(); iter.next()) {
    if (strcmp(currClient, iter.get().name()) == 0) {
      kill_client(CurrBase, iter.get().pid()) ;
      break ;
    }
  }

  ListClients.clear() ;
  iter.reset() ;
  for (; iter.more(); iter.next()) {
    char buff[512] ;
    sprintf(buff, "%s : %d", iter.get().name(), iter.get().pid()) ;    
    ListClients += buff ;
  }
  YOO_WidgetControl->set_choice_text(CLIENT_LIST_WIDGET, ListClients) ;
}

void monitor_quitCB()
{
  exit(0) ;
}


